
/* bu.c - Break up a Terak volume into files */
/* Copyright 1990 by John Foust */
/* July 13, 1990 */

#include <stdio.h>
#include <string.h>

#include "teradecl.h"
#define PRIMARY
#include "teravars.h"


/* Convert Terak text to regular text */

void convTerakToASCII( FILE *infp, FILE *outfp, long size )
{
int each;
int ch;


    while ( (size > 0) && (ch=getc(infp)) != -1) {

        size --;

        /* Handle indents at front of line */
        if (ch == DLE) {

            /* Get indent amount, where 32 means indent 0 */
            ch = getc( infp );
            size --;
            if (ch >= 32) {
                ch = ch - 32;
                for (each=0; each<ch; each++) {
                   putc( ' ', outfp );
                }
            }
        }
        /* If a CR, output LF */
        else if (ch == 0x0D) {
            /* On the PC, emit a CR/LF sequence at the end of lines */
#if defined(_MSC_VER)
            putc( '\r', outfp );
#endif
            /* On other machines, emit just LF */
            putc( '\n', outfp );
        }
        /* Skip nulls, but output everything else */
        else if (ch != 0) {
            putc( ch, outfp );
        }
    }
}


void makeWinNTname( char *str )
{
    while (*str) {
        if (*str == '\\') {
            *str = '_';
        }
        str ++;
    }
}



char wrbuf[ TERAK_BLK_SIZE ];

dirEntry dir[ MAX_DIR_ENTRIES ];     /* 77 * 26 bytes = 2002 */


/* Print a directory of this file */

int main( int argc, char **argv )
{
USHORT lerr;
short each;
short freespace;
short bk;
char inname[128];
FILE *infp;
FILE *outfp;
dirRoot dirRoot;
dirEntry *eachEntry;
char newname[16];


    infp = NULL;
    outfp = NULL;

    if (argc == 2) {
        strcpy( inname, argv[1] );
    }
    else {
        puts( "Bursts files from Terak disk images" );
        puts( "Usage: bu imagefilename" );
        goto out;
    }

    /* Open input file from command line */
    printf( "Bursting Terak disk image file '%s'\n\n", inname );
    infp = fopen( inname, "rb" );
    if (infp == NULL) {
        puts("Couldn't open disk image file.");
        goto out;
    }

    /* Read volume info */
    if ((lerr=readVolInfo( infp, &dirRoot )) != TE_NOERROR) {
        puts( "Failed on volume info." );
        goto out;
    }

    /* Read file directory info */
    if ((lerr=readFileInfo( infp, &dir[1] )) != TE_NOERROR) {
        puts( "Failed on volume info." );
        goto out;
    }

    /* Calculate free space on volume */
    freespace = calcFreeSpace( &dirRoot, &dir[1] );

    /* Print volume info header */
    if ((lerr=printVolInfo( &dirRoot, freespace )) != TE_NOERROR) {
        goto out;
    }

    /* Burst each entry */
    outfp = NULL;
    eachEntry = &dir[1];
    for (each=0; each<dirRoot.numFiles; each++) {

        /* By default, just use the raw filename on this system */
        strcpy( newname, eachEntry->fileName );

        /* Could make an option for Unix to make filename lower-case */
        ;

        /* Translate the filename to something DOS-compatible */
        /* makeDOSname( newname ); */

        /* Translate the filename to something WinNT-compatible */
        makeWinNTname( newname );

        /* For example, Terak allowed '/' in filenames */

        printf( "Making '%-15s'", eachEntry->fileName );

        /* If we changed the name, we might want to show them the new name */
        if (strcmp( newname, eachEntry->fileName) != 0) {
            printf( " as '%-15s'", newname );
        }

        printf( " %s", type_str[ eachEntry->fileKind ] );
        printf( "  %2hd-%s-%2hd", 
                DAY(eachEntry->lastAccess),
                months[ MONTH(eachEntry->lastAccess) ],
                YEAR(eachEntry->lastAccess) );
        printf( "  %4hd %4hd\n", eachEntry->firstBlk, eachEntry->lastBlk );

        outfp = fopen( newname, "wb" );
        if (outfp == NULL) {
            printf( "Couldn't open file '%s' (skipped)\n", newname );
            /* goto out; */
        }

        if (outfp != NULL) {
            int len;
            int looksLikeText;


            /* Move to the beginning of this file */
            if (fseek( infp, (long) eachEntry->firstBlk * TERAK_BLK_SIZE, SEEK_SET ) != 0) {
                puts( "Seek failed." );
                goto out;
            }

            /* If the filename has ".TEXT" at the end, it's a text file */
            /* This gets around Plocher disks with strange directories */
            looksLikeText = 0;
            len = strlen( eachEntry->fileName );
            if (len > 4) {
                if (strcmp( &eachEntry->fileName[ len - 5 ], ".TEXT" ) == 0) {
                    looksLikeText = 1;
                }
            }

            /* For text files, convert them */
            if (eachEntry->fileKind == FK_TEXT || looksLikeText) {

                /* Read past the two header blocks */
                if (fseek( infp, (long) 2 * TERAK_BLK_SIZE, SEEK_CUR ) != 0) {
                    puts( "Seek failed." );
                    goto out;
                }

                /* Could print or store info from the EDITOR header block */

                convTerakToASCII( infp, outfp, 
                    (long) TERAK_BLK_SIZE * (eachEntry->lastBlk - eachEntry->firstBlk - 2) );
            }
            /* Code */
            else if (eachEntry->fileKind == FK_CODE) {
                /* R/W header plus all data */
                for (bk=eachEntry->firstBlk; bk<eachEntry->lastBlk+1; bk++) {
                    if (fread( &wrbuf[0], (size_t) TERAK_BLK_SIZE, (size_t) 1, infp ) != 1) {
                        puts( "Read failed" );
                        goto out;
                    }
                    fwrite( &wrbuf[0], (size_t) TERAK_BLK_SIZE, (size_t) 1, outfp );
                }
            }
            /* Else just write the binary data */
            else {
                /* Read past the header block */
                if (fseek( infp, (long) TERAK_BLK_SIZE, SEEK_CUR ) != 0) {
                    puts( "Seek failed." );
                    goto out;
                }

                /* R/W one less */
                for (bk=eachEntry->firstBlk; bk<eachEntry->lastBlk; bk++) {
                    if (fread( &wrbuf[0], (size_t) TERAK_BLK_SIZE, (size_t) 1, infp ) != 1) {
                        puts( "Read failed" );
                        goto out;
                    }
                    fwrite( &wrbuf[0], (size_t) TERAK_BLK_SIZE, (size_t) 1, outfp );
                }
            }

            fclose( outfp );
            outfp = NULL;
        }

        eachEntry ++;
    }

out:
    if (infp != NULL) {
        fclose( infp );
        infp = NULL;
    }
    if (outfp != NULL) {
        fclose( outfp );
        outfp = NULL;
    }

    return 0;
}

