
/* List a directory of a Terak volume */
/* Copyright 1994 Syndesis Corporation */
/* July 12, 1990 */

#include <stdio.h>
#include <string.h>

#include "teradecl.h"
#define PRIMARY
#include "teravars.h"

/* Improvements:
DOS and ls -l style output,
sort entries by date/size/filename,
add precise byte count of files,
add CRC of files for quick-comparing whether files are same,
ignore standard system files,
function to display which files are missing from disk image
in case it was truncated

Could merge this with the RT-11 reader, and simply
convert the filesystem entries to a common format struct,
and then all the other utility functions could operate
on the same data.

Has bugs on disks like Plocher's "util" disk, where last file
isn't properly converted to a text file?

Also fails when accidentally give TD a RT-11 volume instead.
Merging should auto-detect which disk type is here.

from recover.text:
DUPLICATE DIRECTORIES: Disks should, in general, always have a duplicate
directory.  You can tell if this exists by looking at an extended directory
listing.  The first file of such a disk will start on block 10, otherwise it
will start at block 6.  If your disk does not have a duplicate, and you would
like to add this, you can do so by executing MARKDUPDIR. You must first make
sure that blocks 6-9 on the disk are free (e.g. by transferring the first file
on the disk out of the way).
*/


/* Print a directory of this file */

dirEntry dir[ MAX_DIR_ENTRIES ];     /* 77 * 26 bytes = 2002 */

int main( int argc, char **argv )
{
USHORT lerr;
short each;
short freespace;
char inname[128];
FILE *infp;
dirRoot dirRoot;
dirEntry *eachEntry;
long filesize;
short totalUsed;


    infp = NULL;

    if (argc == 2) {
        strcpy( inname, argv[1] );
    }
    else {
        puts( "Lists directories of Terak disk image files" );
        puts( "Usage: td imagefilename" );
        goto out;
    }

    /* Open input file from command line */
    printf( "Directory of Terak disk image file '%s'\n\n", inname );
    infp = fopen( inname, "rb" );
    if (infp == NULL) {
        puts("Couldn't open file.");
        goto out;
    }

    /* Check size of disk image */
    {
    long odd;
    long blocks;

    fseek( infp, 0, 2 );
    filesize = ftell( infp );
    fseek( infp, 0, 0 );

    /* Check if a natural Terak disk size */
    odd = filesize % 0x0200;
    blocks = filesize / 0x0200;

    /* Double density */
    if (filesize == 583680 && odd == 0 && blocks == 1140) {
        printf( "Double density, " );
    }
    /* Single density */
    else if (filesize == 252928 && odd == 0 && blocks == 494) {
        printf( "Single density, " );
    }
    else {
/*
from format.code:
Single-sided, Single-density (494 blocks)
Single-sided, Double-density (1140 blocks)
Double-sided, Double-density (2280 blocks) */        ;
    }

    printf( "size %ld, odd %ld, blocks %ld of %d bytes each\n", filesize, odd, blocks, 0x0200 );
    }

    /* Read volume info */
    if ((lerr=readVolInfo( infp, &dirRoot )) != TE_NOERROR) {
        puts( "Failed on volume info." );
        goto out;
    }

    /* Read file directory info */
    if ((lerr=readFileInfo( infp, &dir[1] )) != TE_NOERROR) {
        puts( "Failed on volume info." );
        goto out;
    }

    /* Calculate free space on volume */
    freespace = calcFreeSpace( &dirRoot, &dir[1] );

    /* Print volume info header */
    if ((lerr=printVolInfo( &dirRoot, freespace )) != TE_NOERROR) {
        goto out;
    }

    /* Print each entry */
    totalUsed = 0;
    eachEntry = &dir[1];
    for (each=0; each<dirRoot.numFiles; each++) {

        printf( "%3hd: ", each+1 );
        printf( "  %-15s", eachEntry->fileName );
        printf( " %s", type_str[ eachEntry->fileKind ] );
        printf( "  %2d-%s-%2d", 
                DAY(eachEntry->lastAccess),
                months[ MONTH(eachEntry->lastAccess) ],
                YEAR(eachEntry->lastAccess) );

        totalUsed += (eachEntry->lastBlk - eachEntry->firstBlk);

        printf( "  %4d %4d %4d : %5d\n", 
            eachEntry->firstBlk, eachEntry->lastBlk, totalUsed,
            (eachEntry->lastBlk - eachEntry->firstBlk) * 0x200 + eachEntry->lastByte );

        eachEntry ++;
    }

out:
    if (infp != NULL) {
        fclose( infp );
        infp = NULL;
    }

    return 0;
}

