
/* List a directory of a Terak RT-11 volume */
/* Copyright 1994 Syndesis Corporation */
/* July 12, 1990 */

#include <stdio.h>
#include <string.h>

#include "teradecl.h"
#define PRIMARY
#include "teravars.h"


/* Types for rtFileEntry 'status' */

#define RT_TENTATIVE    256     /* Temporary file */
#define RT_EMPTY        512     /* Marks empty space */
#define RT_PERM         1024    /* A "real" file */
#define RT_ENDMARK      2048    /* Marks the end of file entries */


/* Number of file descriptors in the directory block */

#define RT_NUM_FILES    72


/* RT-11 file directory entry, stored in rtDirEntry below */

typedef struct {
    USHORT status;      /* */
    USHORT name[3];     /* Six bytes to store the filename in RAD-50 format */
    /* No element structure padding implied as being in the file here */
    USHORT length;      /* */
    USHORT filler;      /* */
    USHORT date;        /* */
} rtFileEntry;


/* RT-11 volume directory entry, stored in 1024 bytes */

typedef struct {
    USHORT segsAvailable;   /* */
    USHORT nextSegment;     /* */
    USHORT highSegment;     /* */
    USHORT filler;          /* */
    USHORT beginSegment;    /* */
    rtFileEntry files[72];  /* */
} rtDirRoot;


/* Read in a dirRoot struct, the volume info */

USHORT readRT11VolInfo( FILE *infp, rtDirRoot *root )
{
USHORT lerr;
rtFileEntry *ptr;
short each;


    /* Read the volume info into dirRoot, from block 6 and 7 of the disk */
    fseek( infp, (long) 0x0200 * 6, SEEK_SET );

    /* Read the volume info */
    if ((lerr=getShort( infp, &root->segsAvailable )) != TE_NOERROR) {
        goto out;
    }
    if ((lerr=getShort( infp, &root->nextSegment )) != TE_NOERROR) {
        goto out;
    }
    if ((lerr=getShort( infp, &root->highSegment )) != TE_NOERROR) {
        goto out;
    }
    if ((lerr=getShort( infp, &root->filler )) != TE_NOERROR) {
        goto out;
    }
    if ((lerr=getShort( infp, &root->beginSegment )) != TE_NOERROR) {
        goto out;
    }

    ptr = root->files;
    for (each=0; each<RT_NUM_FILES; each++) {

        if ((lerr=getShort( infp, &ptr->status )) != TE_NOERROR) {
            goto out;
        }
        if ((lerr=getShort( infp, &ptr->name[0] )) != TE_NOERROR) {
            goto out;
        }
        if ((lerr=getShort( infp, &ptr->name[1] )) != TE_NOERROR) {
            goto out;
        }
        if ((lerr=getShort( infp, &ptr->name[2] )) != TE_NOERROR) {
            goto out;
        }
        if ((lerr=getShort( infp, &ptr->length )) != TE_NOERROR) {
            goto out;
        }
        if ((lerr=getShort( infp, &ptr->filler )) != TE_NOERROR) {
            goto out;
        }
        if ((lerr=getShort( infp, &ptr->date )) != TE_NOERROR) {
            goto out;
        }

        ptr ++;
    }

    lerr = TE_NOERROR;
out:
    return lerr;
} 


/* Convert one 16-bit Radix-50 three-char to ASCII */

void rad50ToAscii( USHORT rad50, char *str )
{
USHORT each;
USHORT val;
USHORT ord;
char ch;


    /* From each RAD-50 value, we store three ASCII chars */
    for (each=0; each<3; each++) {

        val = rad50;

        /* Which of the three chars we're extracting */
        ord = 2 - (each % 3);
        while (ord-- > 0) {
            val /= 40;
        }
        /* Mask out unused bits */
        val %= 40;

        /* Map 0..39 to ASCII */
        if (val==0)                      ch = ' ';          /* space */
        else if (val >= 1 && val <= 26)  ch = val-1+'A';    /* printable alpha A..Z */
        else if (val == 27)              ch = '$';          /* dollar */
        else if (val == 28)              ch = '.';          /* period */
        else if (val == 29)              ch = ' ';          /* unused char, emit a space instead */
        else if (val >= 30 && val <= 39) ch = val-30+'0';   /* digit 0..9 */

        *str = ch;

        str ++;
    }

    /* Yes, we don't zero-terminate */
}


/* Convert RT-11 date to ASCII "01-May-79" */

static char *month_str[] = 
    { "???", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

void rtDateStr( USHORT date, char *str )
{
int day;
int month;
int year;


    year  = (date & 0x1F) + 72;
    day   = (date >> 5)  & 0x1F;
    month = (date >> 10) & 0x1F;

    if (month < 1 || month > 12) {
        month = 0;
    }

    sprintf( str, "%02d-%3s-%02d", day, month_str[month], year );
}


int main( int argc, char **argv )
{
USHORT lerr;
int each;
char inname[128];
FILE *infp;
rtDirRoot dirRoot;
rtFileEntry *eachEntry;
char buff[ 16 ];
USHORT block;
char *lineSpec;


    infp = NULL;

    if (argc == 2) {
        strcpy( inname, argv[1] );
    }
    else {
        puts( "Lists directories of Terak RT-11 disk image files" );
        puts( "Usage: tdrt imagefilename" );
        goto out;
    }

    /* Open input file from command line */
    printf( "Directory of Terak RT-11 disk image file '%s'\n\n", inname );
    infp = fopen( inname, "rb" );
    if (infp == NULL) {
        puts("Couldn't open file.");
        goto out;
    }

    /* Read volume info, including 72 file entries */
    if ((lerr=readRT11VolInfo( infp, &dirRoot )) != TE_NOERROR) {
        puts( "Failed on volume info." );
        goto out;
    }

    /* Print volume info header */
    printf( "segs %hu, next %hu, high %hu, filler %hu, begin %hu\n\n",
        dirRoot.segsAvailable, dirRoot.nextSegment, dirRoot.highSegment,
        dirRoot.filler, dirRoot.beginSegment );

    /* Header */
    printf( "              Size  Block   Date\n" );

    /* Format specifier for each line */
    lineSpec = "  %-11s  %3hu  %4hu";

    /* Print each entry */
    block = dirRoot.beginSegment;
    eachEntry = &dirRoot.files[0];
    for (each=0; each<RT_NUM_FILES; each++) {

        /* If no more files */
        if (eachEntry->status == RT_ENDMARK) {
            break;
        }
        /* If a temporary file */
        else if (eachEntry->status == RT_TENTATIVE) {
            printf( lineSpec, "<temp>", eachEntry->length, block );
            /* Always have date of 0 */
            printf( "\n" );
        }
        /* If empty space, like at end of list */
        else if (eachEntry->status == RT_EMPTY) {
            printf( lineSpec, "<empty>", eachEntry->length, block );
            /* Always have date of 0 */
            printf( "\n" );
        }
        /* If a permanent file */
        else if (eachEntry->status == RT_PERM) {

            /* Convert RAD-50 filename to ASCII, including extension '.' */
            rad50ToAscii( eachEntry->name[0], &buff[0] );
            rad50ToAscii( eachEntry->name[1], &buff[3] );
            buff[6] = '.';
            rad50ToAscii( eachEntry->name[2], &buff[7] );
            buff[10] = '\0';

            printf( lineSpec, buff, eachEntry->length, block );

            /* Date */
            rtDateStr( eachEntry->date, buff );
            printf( "  %s\n", buff );
        }
        else {
            printf( "Unknown status %hu 0x%04hx\n", eachEntry->status, eachEntry->status );
        }

        /* Keep track of which block we're at */
        block += eachEntry->length;

        eachEntry ++;
    }

out:
    if (infp != NULL) {
        fclose( infp );
        infp = NULL;
    }

    return 0;
}

